/*
 * Copyright (c) 2016 Nordic Semiconductor ASA
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 *   3. Neither the name of Nordic Semiconductor ASA nor the names of other
 *   contributors to this software may be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 *   4. This software must only be used in or with a processor manufactured by Nordic
 *   Semiconductor ASA, or in or with a processor manufactured by a third party that
 *   is used in combination with a processor manufactured by Nordic Semiconductor.
 *
 *   5. Any software provided in binary or object form under this license must not be
 *   reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef ADAPTER_INTERNAL_H__
#define ADAPTER_INTERNAL_H__

#include "sd_rpc_types.h"
#include "serialization_transport.h"

#include "ble.h"
#include "nrf_error.h"

#include <string>

class AdapterInternal
{
  public:
    explicit AdapterInternal(SerializationTransport *transport);
    ~AdapterInternal();
    uint32_t open(const sd_rpc_status_handler_t status_callback,
                  const sd_rpc_evt_handler_t event_callback,
                  const sd_rpc_log_handler_t log_callback);
    uint32_t close();
    uint32_t logSeverityFilterSet(const sd_rpc_log_severity_t severity_filter);
    static bool isInternalError(const uint32_t error_code);

    void statusHandler(const sd_rpc_app_status_t code, const std::string &error);
    void eventHandler(ble_evt_t *event);
    void logHandler(const sd_rpc_log_severity_t severity, const std::string &log_message);

    SerializationTransport *transport;

  private:
    sd_rpc_evt_handler_t eventCallback;
    sd_rpc_status_handler_t statusCallback;
    sd_rpc_log_handler_t logCallback;
    sd_rpc_log_severity_t logSeverityFilter;

    bool isOpen;
    std::mutex publicMethodMutex;
};

#endif // ADAPTER_INTERNAL_H__
