/*
 * Copyright (c) 2016 Nordic Semiconductor ASA
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 *   3. Neither the name of Nordic Semiconductor ASA nor the names of other
 *   contributors to this software may be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 *   4. This software must only be used in or with a processor manufactured by Nordic
 *   Semiconductor ASA, or in or with a processor manufactured by a third party that
 *   is used in combination with a processor manufactured by Nordic Semiconductor.
 *
 *   5. Any software provided in binary or object form under this license must not be
 *   reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef BLE_COMMON_IMPL_H__
#define BLE_COMMON_IMPL_H__

#include "adapter.h"
#include <functional>
#include <stdint.h>

typedef std::function<uint32_t(uint8_t *, uint32_t *)> encode_function_t;
typedef std::function<uint32_t(uint8_t *, uint32_t, uint32_t *)> decode_function_t;

uint32_t encode_decode(adapter_t *adapter, const encode_function_t &encode_function,
                       const decode_function_t &decode_function);

/*
 * We do not want to change the codecs provided by the SDK too much. The BLESecurityContext provides
 * a way to set the root security context before calling the codecs. Typically the root context is
 * the SerializationTransport object.
 */

class RequestReplyCodecContext
{
  public:
    explicit RequestReplyCodecContext(void *adapterId);
    ~RequestReplyCodecContext();
    RequestReplyCodecContext(const RequestReplyCodecContext &) = delete;
    RequestReplyCodecContext &operator=(const RequestReplyCodecContext &) = delete;
    RequestReplyCodecContext(RequestReplyCodecContext &&)                 = delete;
    RequestReplyCodecContext &operator=(RequestReplyCodecContext &&) = delete;
};

class EventCodecContext
{
  public:
    explicit EventCodecContext(void *adapterId);
    ~EventCodecContext();
    EventCodecContext(const EventCodecContext &) = delete;
    EventCodecContext &operator=(const EventCodecContext &) = delete;
    EventCodecContext(EventCodecContext &&)                 = delete;
    EventCodecContext &operator=(EventCodecContext &&) = delete;
};

#endif
